// Header files of frequent usage

#include <algorithm>
#include <cmath>
#include <cstring>
#include <vector>
#include <iostream>
#include <sstream>
#include <queue>
#include <map>
#include <set>

#include <GL/glew.h>
#include <SDL2/SDL.h>

// Some useful defines

#define FOR(q,n) for(int q=0;q<n;q++)
#define SFOR(q,s,e) for(int q=s;q<=e;q++)
#define RFOR(q,n) for(int q=n;q>=0;q--)
#define RSFOR(q,s,e) for(int q=s;q>=e;q--)

#define ESZ(elem) (int)elem.size()

#ifdef USE_MESSAGEBOX
	#define PRINT_ERROR(x) { std::stringstream ss; ss << x; SDL_ShowSimpleMessageBox(SDL_MESSAGEBOX_ERROR, "Error!", ss.str().c_str(), NULL); }
	// You probably don't want a messagebox popping up for every info print
//	#define PRINT_INFO(x) { std::stringstream ss; ss << x; SDL_ShowSimpleMessageBox(SDL_MESSAGEBOX_INFORMATION, "Info", ss.str().c_str(), NULL); }
	#define PRINT_INFO(x) { std::cout << x << std::endl; }
#else
	#define PRINT_ERROR(x) { std::cerr << x << '\n'; }
	#define PRINT_INFO(x) { std::cout << x << std::endl; }
#endif

// Sprinkle calls to this function liberally to track down OpenGL errors
inline GLenum CheckGLError(const char * str = "")
{
	GLenum error = glGetError();
	if (error)
		PRINT_ERROR(str << " - OpenGL Error: 0x" << std::hex << error);
	return error;
}

// Typedefs for Windows compatibility
typedef unsigned int UINT;
typedef void * LPVOID;

const int INITIAL_WINDOW_WIDTH = 1024;
const int INITIAL_WINDOW_HEIGHT = 700;
