#include "common_header.h"
#include "OpenGLApp.h"

COpenGLWinApp appMain;

/*-----------------------------------------------

Name:   CreateAppWindow

Params: sTitle - title of created window

Result: Creates main application window.

/*---------------------------------------------*/

bool COpenGLWinApp::CreateAppWindow(std::string sTitle)
{
	if (!oglControl.InitSDL2(3, 3))
		return false;

	window = SDL_CreateWindow(sTitle.c_str(), SDL_WINDOWPOS_UNDEFINED,
		SDL_WINDOWPOS_UNDEFINED, INITIAL_WINDOW_WIDTH, INITIAL_WINDOW_HEIGHT,
		SDL_WINDOW_OPENGL | SDL_WINDOW_RESIZABLE);

	if (!oglControl.InitOpenGL(window, InitScene, RenderScene, NULL, &oglControl))
		return false;

	return true;
}

/*-----------------------------------------------

Name:   AppBody

Params: none

Result: Main application body infinite loop.

/*---------------------------------------------*/

void COpenGLWinApp::AppBody()
{
	SDL_Event event;
	int quit = 0;
	while (!quit) {
		while (SDL_PollEvent(&event)) {
			if (event.type == SDL_QUIT)
				quit = 1;
			else
				MsgHandlerMain(&event);
		}
		if (bAppActive) {
			oglControl.Render(&oglControl);
		}
		else
			SDL_Delay(200);
	}
}

/*-----------------------------------------------

Name:   Shutdown

Params: none

Result: Shuts down application and releases used
        memory.

/*---------------------------------------------*/

void COpenGLWinApp::Shutdown()
{
	oglControl.ReleaseOpenGLControl(&oglControl);
}

/*-----------------------------------------------

Name:   MsgHandlerMain

Params: windows message stuff

Result: Application messages handler.

/*---------------------------------------------*/

void COpenGLWinApp::MsgHandlerMain(SDL_Event * event)
{
	switch(event->type) {
		case SDL_WINDOWEVENT:
			switch(event->window.event) {
				case SDL_WINDOWEVENT_SIZE_CHANGED:
					oglControl.ResizeOpenGLViewportFull();
					break;
				case SDL_WINDOWEVENT_FOCUS_GAINED:
					bAppActive = true;
					break;
				case SDL_WINDOWEVENT_FOCUS_LOST:
					bAppActive = false;
					break;
				case SDL_WINDOWEVENT_EXPOSED:
					if (!bAppActive)
						oglControl.SwapBuffersM();
					break;
			}
			break;
	}
}

/*-----------------------------------------------

Name:   WinMain

Params: whatever

Result: Windows entry point for application.

/*---------------------------------------------*/

int main(int argc, char ** argv)
{
	PRINT_INFO("Starting Application");

	if(!appMain.CreateAppWindow("03.) Shaders Are Coming - Tutorial by Michal Bubnar (www.mbsoftworks.sk)"))
		return 1;

	appMain.AppBody();
	appMain.Shutdown();

	return 0;
}
